package betterwithmods.common.blocks;

import betterwithmods.common.BWMBlocks;
import betterwithmods.util.DirUtils;
import com.google.common.collect.Maps;
import net.minecraft.block.Block;
import net.minecraft.block.material.EnumPushReaction;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.EnumSkyBlock;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.List;
import java.util.Random;

public class BlockLens extends BWMBlock {
    public static final PropertyBool LIT = PropertyBool.func_177716_a("lit");
    public static final int RANGE = 256;

    public BlockLens() {
        super(Material.field_151573_f);
        this.func_149711_c(3.5F);
        this.func_149675_a(true);
        this.func_180632_j(this.func_176223_P().func_177226_a(DirUtils.FACING, EnumFacing.NORTH).func_177226_a(LIT, false));
        this.setHarvestLevel("pickaxe", 0);
    }


    @Override
    public int func_149738_a(World world) {
        return 5;
    }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        world.func_180497_b(pos, this, 3, 5);
    }

    @Nonnull
    @Override
    public IBlockState getStateForPlacement(@Nonnull World world, @Nonnull BlockPos pos, @Nonnull EnumFacing side, float flX, float flY, float flZ, int meta, @Nonnull EntityLivingBase entity, EnumHand hand) {
        IBlockState state = super.getStateForPlacement(world, pos, side, flX, flY, flZ, meta, entity, hand);
        EnumFacing face = DirUtils.convertEntityOrientationToFacing(entity, side);
        return setFacingInBlock(state, face);
    }

    @Override
    public void func_180663_b(@Nonnull World world, @Nonnull BlockPos pos, @Nonnull IBlockState state) {
        cleanupLightToFacing(world, pos, state.func_177229_b(DirUtils.FACING));
        super.func_180663_b(world, pos, state);
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase entity, ItemStack stack) {
        EnumFacing facing = DirUtils.convertEntityOrientationToFacing(entity, EnumFacing.NORTH);
        setFacingInBlock(state, facing);
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_189540_a(IBlockState state, World world, BlockPos pos, Block block, BlockPos other) {
        world.func_180497_b(pos, this, 3, 5);
    }

    private boolean canPassLight(World world, BlockPos pos, IBlockState state) {
        return !state.func_185904_a().func_76218_k() || state.func_177230_c().isAir(state, world, pos);
    }

    private void placeLightSource(World world, BlockPos pos, EnumFacing facing, BlockPos lensPos) {
        if (world.func_180495_p(pos).func_177230_c().func_176200_f(world, pos)) {
            boolean sunlight = isLightFromSun(world, lensPos);
            world.func_175656_a(pos, BWMBlocks.LIGHT_SOURCE.func_176223_P().func_177226_a(BlockInvisibleLight.SUNLIGHT, sunlight).func_177226_a(DirUtils.FACING, facing));
        }
    }


    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand) {
        cleanupLight(world, pos);
        EnumFacing dir = getFacing(world, pos), opposite = dir.func_176734_d();

        boolean lightOn = isInputLit(world, pos);
        if (isLit(world, pos) != lightOn) {
            setLit(world, pos, lightOn);
        }

        if (isLit(world, pos)) {
            int currentRange = RANGE;
            for (int i = 1; i < RANGE; i++) {
                BlockPos offset = pos.func_177967_a(dir, i);
                IBlockState offsetState = world.func_180495_p(offset);

                if (!canPassLight(world, offset, offsetState)) {
                    currentRange = i + 1;
                    break;
                }
            }

            AxisAlignedBB bb = field_185505_j.func_186670_a(pos);

            int mod = dir.func_176743_c().func_179524_a();
            switch (dir.func_176740_k()) {
                case X:
                    bb = bb.func_72321_a(mod * currentRange, 0, 0);
                    break;
                case Y:
                    bb = bb.func_72321_a(0, mod * currentRange, 0);
                    break;
                case Z:
                    bb = bb.func_72321_a(0, 0, mod * currentRange);
                    break;
            }


            List<Entity> box = world.func_72872_a(Entity.class, bb);
            HashMap<Integer, Entity> map = Maps.newHashMap();
            for (Entity e : box) {
                int distance = 0;
                switch (dir.func_176740_k()) {
                    case X:
                        distance = (int) (pos.func_177958_n() - e.field_70165_t);
                        break;
                    case Y:
                        distance = (int) (pos.func_177956_o() - e.field_70163_u);
                        break;
                    case Z:
                        distance = (int) (pos.func_177952_p() - e.field_70161_v);
                        break;
                }
                map.put(Math.abs(distance), e);
            }

            for (int i = 1; i < currentRange; i++) {
                BlockPos offset = pos.func_177967_a(dir, i);
                IBlockState offsetState = world.func_180495_p(offset);
                boolean blocked = false;
                if (canPassLight(world, pos, offsetState)) {
                    if (map.containsKey(i)) {
                        blocked = true;
                    }
                } else {
                    blocked = true;
                }

                if (blocked) {
                    placeLightSource(world, offset.func_177972_a(opposite), opposite, pos);
                    break;
                }
            }

        }

        world.func_180497_b(pos, this, 5, 5);
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public EnumPushReaction func_149656_h(IBlockState state) {
        return EnumPushReaction.BLOCK;
    }

    public EnumFacing getFacing(IBlockAccess world, BlockPos pos) {
        return getFacing(world.func_180495_p(pos));
    }

    public EnumFacing getFacing(IBlockState state) {
        return state.func_177229_b(DirUtils.FACING);
    }

    public IBlockState setFacingInBlock(IBlockState state, EnumFacing facing) {
        return state.func_177226_a(DirUtils.FACING, facing);
    }

    public boolean isLit(IBlockAccess world, BlockPos pos) {
        return world.func_180495_p(pos).func_177229_b(LIT);
    }

    public void setLit(World world, BlockPos pos, boolean isOn) {
        boolean oldLit = world.func_180495_p(pos).func_177229_b(LIT);

        if (isOn != oldLit) {
            world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(LIT, isOn));
            world.func_190524_a(pos, this, pos);
        }
    }

    private boolean isInputLit(World world, BlockPos pos) {
        EnumFacing facing = getFacing(world, pos);
        EnumFacing dir = DirUtils.getOpposite(facing);
        BlockPos oppOff = pos.func_177972_a(dir);
        Block block = world.func_180495_p(oppOff).func_177230_c();

        if (world.func_175623_d(oppOff) && world.func_175710_j(oppOff)) {
            return world.func_175642_b(EnumSkyBlock.SKY, oppOff) > 12 && world.func_72935_r() && (!world.func_72896_J() || !world.func_72911_I());
        } else {
            return block.getLightValue(world.func_180495_p(oppOff), world, oppOff) > 12;
        }
    }


    private boolean isLightFromSun(World world, BlockPos pos) {
        EnumFacing facing = DirUtils.getOpposite(getFacing(world, pos));
        BlockPos offset = pos.func_177972_a(facing);
        if (world.func_175623_d(offset) && world.func_175710_j(offset))
            return true;
        else {
            return world.func_180495_p(offset).func_177230_c() == BWMBlocks.LIGHT_SOURCE && world.func_180495_p(offset).func_177229_b(BlockInvisibleLight.SUNLIGHT);
        }
    }

    private void cleanupLightToFacing(World world, BlockPos pos, EnumFacing facing) {
        EnumFacing opp = DirUtils.getOpposite(facing);
        for (int i = 1; i < RANGE; i++) {
            BlockPos offset = pos.func_177967_a(facing, i);
            if (world.func_180495_p(offset).func_177230_c() == BWMBlocks.LIGHT_SOURCE) {
                EnumFacing lightFace = ((BlockInvisibleLight) world.func_180495_p(offset).func_177230_c()).getFacing(world, offset);
                if (lightFace == opp) {
                    world.func_175698_g(offset);
                    break;
                }
            } else if (!world.func_175623_d(offset))
                break;
        }
    }

    private void cleanupLight(World world, BlockPos pos) {
        EnumFacing facing = getFacing(world, pos);
        EnumFacing oppFacing = DirUtils.getOpposite(facing);

        for (int i = 1; i < RANGE; i++) {
            BlockPos offset = pos.func_177967_a(facing, i);

            if (world.func_180495_p(offset).func_177230_c() == BWMBlocks.LIGHT_SOURCE) {
                EnumFacing lightFace = ((BlockInvisibleLight) world.func_180495_p(offset).func_177230_c()).getFacing(world, offset);
                if (lightFace == oppFacing) {
                    world.func_175698_g(offset);
                }
            } else if (!world.func_175623_d(offset)) {
                if (world.func_180495_p(offset).func_177230_c() == this) {
                    BlockLens lens = (BlockLens) world.func_180495_p(pos).func_177230_c();
                    if (lens.getFacing(world, offset) == facing)
                        break;
                }
            }
        }
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta) {
        boolean lit = false;
        if (meta > 7) {
            lit = true;
            meta -= 8;
        }
        return this.func_176223_P().func_177226_a(LIT, lit).func_177226_a(DirUtils.FACING, EnumFacing.func_82600_a(meta));
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int meta = state.func_177229_b(LIT) ? 8 : 0;
        return meta + state.func_177229_b(DirUtils.FACING).func_176745_a();
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, DirUtils.FACING, LIT);
    }

    @Override
    public void nextState(World world, BlockPos pos, IBlockState state) {
        world.func_175656_a(pos, state.func_177226_a(LIT, false).func_177231_a(DirUtils.FACING));
    }

    @Override
    public boolean rotates() {
        return true;
    }
}
