package betterwithmods.common.blocks;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.BlockRenderLayer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.IStringSerializable;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.Random;

public abstract class BlockStickBase extends BWMBlock {
    public static final PropertyEnum<Connection> CONNECTION = PropertyEnum.func_177709_a("connection", Connection.class);
    public static final PropertyBool GROUND = PropertyBool.func_177716_a("ground");

    public BlockStickBase(Material material) {
        super(material);
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, CONNECTION, GROUND);
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return 0;
    }

    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta) {
        boolean ground = (meta & 1) == 1;
        int connection = meta >> 1;
        return func_176223_P().func_177226_a(GROUND, ground).func_177226_a(CONNECTION, Connection.VALUES[connection]);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int connection = state.func_177229_b(CONNECTION).ordinal() << 1;
        int ground = state.func_177229_b(GROUND) ? 1 : 0;
        return ground | connection;
    }

    public abstract IBlockState getConnections(IBlockState state, IBlockAccess world, BlockPos pos);

    public abstract double getHeight(IBlockState state);

    @Nonnull
    @Override
    public IBlockState func_176221_a(@Nonnull IBlockState state, IBlockAccess worldIn, BlockPos pos) {
        IBlockState newState = state;
        if (worldIn.func_180495_p(pos.func_177977_b()).isSideSolid(worldIn, pos, EnumFacing.UP)) {
            newState = newState.func_177226_a(GROUND, true);
        }
        return getConnections(newState, worldIn, pos);
    }

    @Override
    public boolean canPlaceTorchOnTop(IBlockState state, @Nonnull IBlockAccess world, @Nonnull BlockPos pos) {
        return true;
    }

    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess p_193383_1_, IBlockState p_193383_2_, BlockPos p_193383_3_, EnumFacing face) {
        return face.func_176740_k() == EnumFacing.Axis.Y ? BlockFaceShape.CENTER_SMALL : BlockFaceShape.UNDEFINED;
    }

    @Nonnull
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k() {
        return BlockRenderLayer.CUTOUT;
    }

    @Nonnull
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
        double y = getHeight(state);
        return new AxisAlignedBB(6.5d / 16d, 0, 6.5d / 16d, 9.5d / 16d, y, 9.5d / 16d);
    }

    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos) {
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
        this.checkAndDropBlock(worldIn, pos, state);
    }

    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand) {
        this.checkAndDropBlock(worldIn, pos, state);
    }

    private void checkAndDropBlock(World worldIn, BlockPos pos, IBlockState state) {
        if (!this.func_176196_c(worldIn, pos)) {
            this.func_176226_b(worldIn, pos, state, 0);
            worldIn.func_175698_g(pos);
        }
    }

    public enum Connection implements IStringSerializable {
        DISCONNECTED,
        CONNECTED,
        CANDLE,
        SKULL;

        public static final Connection[] VALUES = values();

        @Override
        public String func_176610_l() {
            return name().toLowerCase();
        }
    }
}
