package betterwithmods.common.blocks.behaviors;

import betterwithmods.module.hardcore.world.HCBuckets;
import betterwithmods.util.FluidUtils;
import net.minecraft.block.BlockDispenser;
import net.minecraft.dispenser.BehaviorDefaultDispenseItem;
import net.minecraft.dispenser.IBlockSource;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntityDispenser;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidActionResult;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.FluidUtil;
import net.minecraftforge.fluids.capability.IFluidHandlerItem;

import javax.annotation.Nonnull;

public class BehaviorFluidContainer extends BehaviorDefaultDispenseItem {

    private static final BehaviorFluidContainer INSTANCE = new BehaviorFluidContainer();
    private final BehaviorDefaultDispenseItem dispenseBehavior = new BehaviorDefaultDispenseItem();

    private BehaviorFluidContainer() {
    }

    public static BehaviorFluidContainer getInstance() {
        return INSTANCE;
    }

    /**
     * Dispense the specified stack, play the dispense sound and spawn particles.
     */
    @Override
    @Nonnull
    public ItemStack func_82487_b(@Nonnull IBlockSource source, @Nonnull ItemStack stack) {
        if (FluidUtil.getFluidContained(stack) != null) {
            return dumpContainer(source, stack);
        } else {
            if (HCBuckets.stopDispenserFillBehavior) {
                return super.func_82487_b(source, stack);
            }
            return fillContainer(source, stack);
        }
    }

    /**
     * Picks up fluid in front of a Dispenser and fills a container with it.
     */
    @Nonnull
    private ItemStack fillContainer(@Nonnull IBlockSource source, @Nonnull ItemStack stack) {

        World world = source.func_82618_k();
        EnumFacing dispenserFacing = source.func_189992_e().func_177229_b(BlockDispenser.field_176441_a);
        BlockPos blockpos = source.func_180699_d().func_177972_a(dispenserFacing);

        FluidActionResult actionResult = FluidUtils.tryPickUpFluid(stack, null, world, blockpos, dispenserFacing.func_176734_d());
        ItemStack resultStack = actionResult.getResult();

        if (!actionResult.isSuccess() || resultStack.func_190926_b()) {
            return super.func_82487_b(source, stack);
        }

        if (stack.func_190916_E() == 1) {
            return resultStack;
        } else if (((TileEntityDispenser) source.func_150835_j()).func_146019_a(resultStack) < 0) {
            this.dispenseBehavior.func_82482_a(source, resultStack);
        }

        ItemStack stackCopy = stack.func_77946_l();
        stackCopy.func_190918_g(1);
        return stackCopy;
    }

    /**
     * Drains a filled container and places the fluid in front of the Dispenser.
     */
    @Nonnull
    private ItemStack dumpContainer(IBlockSource source, @Nonnull ItemStack stack) {
        ItemStack singleStack = stack.func_77946_l();
        singleStack.func_190920_e(1);
        IFluidHandlerItem fluidHandler = FluidUtil.getFluidHandler(singleStack);
        if (fluidHandler == null) {
            return super.func_82487_b(source, stack);
        }

        FluidStack fluidStack = fluidHandler.drain(Fluid.BUCKET_VOLUME, false);
        EnumFacing dispenserFacing = source.func_189992_e().func_177229_b(BlockDispenser.field_176441_a);
        BlockPos blockpos = source.func_180699_d().func_177972_a(dispenserFacing);
        FluidActionResult result = fluidStack != null ? FluidUtils.tryPlaceFluid(null, source.func_82618_k(), blockpos, stack, fluidStack) : FluidActionResult.FAILURE;

        if (result.isSuccess()) {
            ItemStack drainedStack = result.getResult();

            if (drainedStack.func_190916_E() == 1) {
                return drainedStack;
            } else if (!drainedStack.func_190926_b() && ((TileEntityDispenser) source.func_150835_j()).func_146019_a(drainedStack) < 0) {
                this.dispenseBehavior.func_82482_a(source, drainedStack);
            }

            ItemStack stackCopy = drainedStack.func_77946_l();
            stackCopy.func_190918_g(1);
            return stackCopy;
        } else {
            return this.dispenseBehavior.func_82482_a(source, stack);
        }
    }
}
