package betterwithmods.common.blocks.camo;

import betterwithmods.client.BWCreativeTabs;
import betterwithmods.client.baking.UnlistedPropertyGeneric;
import betterwithmods.common.blocks.BWMBlock;
import betterwithmods.common.tile.TileCamo;
import betterwithmods.module.gameplay.miniblocks.ItemCamo;
import betterwithmods.module.gameplay.miniblocks.MiniBlocks;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Enchantments;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.Explosion;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.property.ExtendedBlockState;
import net.minecraftforge.common.property.IExtendedBlockState;
import net.minecraftforge.common.property.IUnlistedProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;

public abstract class BlockCamo extends BWMBlock {

    public static final IUnlistedProperty<CamoInfo> CAMO_INFO = new UnlistedPropertyGeneric<>("camo", CamoInfo.class);
    private final Function<Material, Collection<IBlockState>> subtypes;

    public BlockCamo(Material material, Function<Material, Collection<IBlockState>> subtypes) {
        super(material);
        this.subtypes = subtypes;
        func_149647_a(BWCreativeTabs.MINI_BLOCKS);
    }

    public Optional<? extends TileCamo> getTile(IBlockAccess world, BlockPos pos) {
        TileEntity tile = world.func_175625_s(pos);
        if (tile instanceof TileCamo)
            return Optional.of((TileCamo) tile);
        return Optional.empty();
    }

    public ItemBlock createItemBlock(Block block) {
        return new ItemCamo(block);
    }

    @SuppressWarnings("deprecation")
    @Override
    public float func_176195_g(IBlockState blockState, World worldIn, BlockPos pos) {
        return getTile(worldIn, pos).map(t -> t.getState().func_185887_b(worldIn, pos)).orElse(super.func_176195_g(blockState, worldIn, pos));
    }


    @Override
    public float getExplosionResistance(World world, BlockPos pos, @Nullable Entity exploder, Explosion explosion) {
        return getTile(world, pos).map(t -> t.getState().func_177230_c().getExplosionResistance(world, pos, exploder, explosion)).orElse(super.getExplosionResistance(world, pos, exploder, explosion));
    }


    @Override
    public void func_149666_a(CreativeTabs itemIn, NonNullList<ItemStack> items) {
        for (IBlockState state : subtypes.apply(field_149764_J)) {
            items.add(MiniBlocks.fromParent(this, state));
        }
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return 0;
    }

    @Nonnull
    @SuppressWarnings("deprecation")
    @Override
    public IBlockState func_176203_a(int meta) {
        return func_176223_P();
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e() {
        return new ExtendedBlockState(this, new IProperty[0], new IUnlistedProperty[]{CAMO_INFO});
    }

    @Nullable
    @Override
    public abstract TileEntity createTileEntity(@Nonnull World world, @Nonnull IBlockState state);

    @Override
    public boolean hasTileEntity(IBlockState state) {
        return true;
    }

    @Nonnull
    @Override
    public IBlockState getExtendedState(@Nonnull IBlockState state, IBlockAccess world, BlockPos pos) {
        IExtendedBlockState extendedBS = (IExtendedBlockState) super.getExtendedState(state, world, pos);
        return getTile(world, pos).map(t -> fromTile(extendedBS, t)).orElse(extendedBS);
    }

    public IBlockState fromTile(IExtendedBlockState state, TileCamo tile) {
        return state.withProperty(CAMO_INFO, new CamoInfo(tile));
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase placer, ItemStack stack) {
    }

    @Nonnull
    @SuppressWarnings("deprecation")
    @Override
    public ItemStack func_185473_a(World worldIn, BlockPos pos, @Nonnull IBlockState state) {
        TileEntity tile = worldIn.func_175625_s(pos);
        if (tile instanceof TileCamo) {
            TileCamo mini = (TileCamo) tile;
            return mini.getPickBlock(null, null, state);
        }
        return new ItemStack(this);
    }


    @Override
    public final void getDrops(@Nonnull NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, @Nonnull IBlockState state, int fortune) {
        getDrops(drops, world, pos, state, null, fortune, false);
    }


    @Override
    public void func_180657_a(@Nonnull World worldIn, EntityPlayer player, @Nonnull BlockPos pos, @Nonnull IBlockState state, @Nullable TileEntity te, ItemStack stack) {
        player.func_71029_a(Objects.requireNonNull(StatList.func_188055_a(this)));
        player.func_71020_j(0.005F);
        int fortuneLevel = EnchantmentHelper.func_77506_a(Enchantments.field_185308_t, stack);
        boolean silkTouch = EnchantmentHelper.func_77506_a(Enchantments.field_185306_r, stack) > 0;

        NonNullList<ItemStack> items = NonNullList.func_191196_a();
        getDrops(items, worldIn, pos, state, te, 0, false);
        float chance = net.minecraftforge.event.ForgeEventFactory.fireBlockHarvesting(items, worldIn, pos, state, fortuneLevel, 1.0f, silkTouch, player);

        harvesters.set(player);

        if (!worldIn.field_72995_K && !worldIn.restoringBlockSnapshots) {
            for (ItemStack item : items)
                if (chance >= 1.0f || worldIn.field_73012_v.nextFloat() <= chance)
                    func_180635_a(worldIn, pos, item);
        }

        harvesters.set(null);
    }

    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, @Nullable TileEntity te, int fortune, boolean silkTouch) {
        if (te instanceof TileCamo) {
            drops.add(((TileCamo) te).getPickBlock(null, null, state));
        } else {
            super.getDrops(drops, world, pos, state, fortune);
        }
    }


    @Override
    public void func_180653_a(World worldIn, @Nonnull BlockPos pos, @Nonnull IBlockState state, float chance, int fortune) {
        if (!worldIn.field_72995_K && !worldIn.restoringBlockSnapshots) {
            NonNullList<ItemStack> drops = NonNullList.func_191196_a();
            getDrops(drops, worldIn, pos, state, worldIn.func_175625_s(pos), fortune, false);
            chance = net.minecraftforge.event.ForgeEventFactory.fireBlockHarvesting(drops, worldIn, pos, state, fortune, chance, false, harvesters.get());

            for (ItemStack drop : drops) {
                if (worldIn.field_73012_v.nextFloat() <= chance) {
                    func_180635_a(worldIn, pos, drop);
                }
            }
        }
    }


    @Nonnull
    @Override
    public ItemStack getPickBlock(@Nonnull IBlockState state, RayTraceResult target, @Nonnull World world, @Nonnull BlockPos pos, EntityPlayer player) {
        return getTile(world, pos).map(t -> t.getPickBlock(player, target, state)).orElse(new ItemStack(this));
    }


    @Override
    public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face) {
        return getTile(world, pos).map(t -> t.getState().func_177230_c().getFireSpreadSpeed(world, pos, face)).orElse(5);
    }

    @Override
    public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face) {
        return getTile(world, pos).map(t -> t.getState().func_177230_c().getFlammability(world, pos, face)).orElse(10);
    }

}
