package betterwithmods.common.items.itemblocks;

import betterwithmods.api.block.IRenderRotationPlacement;
import betterwithmods.common.blocks.mechanical.BlockAxle;
import betterwithmods.common.blocks.mechanical.BlockAxleGenerator;
import betterwithmods.util.DirUtils;
import betterwithmods.util.TooltipLib;
import betterwithmods.util.WorldUtils;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.Objects;

public abstract class ItemAxleBase extends ItemBlock implements IRenderRotationPlacement {
    protected int radius;

    public ItemAxleBase(Block block) {
        super(block);

        this.func_77625_d(1);
    }

    public abstract boolean isAxis(EnumFacing.Axis axis);

    private void showErrorMessage(EntityPlayer player, Error error) {
        String block = Objects.requireNonNull(getRegistryName()).func_110623_a();
        player.func_145747_a(TooltipLib.getMessageComponent(error.format(block)));
    }

    public AxisAlignedBB getBounds(EnumFacing.Axis axis, int radius) {
        switch (axis) {
            case X:
                return new AxisAlignedBB(0, -radius, -radius, 0, radius, radius);
            case Z:
                return new AxisAlignedBB(-radius, -radius, 0, radius, radius, 0);
            default:
                return Block.field_185506_k;
        }
    }

    private boolean containsOtherGenerator(World world, BlockPos center, EnumFacing.Axis axis) {
        int d = radius * 2;
        AxisAlignedBB box = getBounds(DirUtils.rotateAroundY(axis), d);
        Iterable<BlockPos> positions = WorldUtils.getPosInBox(box.func_186670_a(center));
        for (BlockPos p : positions) {
            IBlockState state = world.func_180495_p(p);
            if (state.func_177230_c() instanceof BlockAxleGenerator) {
                return true;
            }
        }
        return false;
    }

    private boolean isValidArea(World world, EntityPlayer player, BlockPos pos, EnumFacing.Axis axis) {
        AxisAlignedBB box = getBounds(axis, radius);
        if (box == null)
            return false;
        Iterable<BlockPos> positions = WorldUtils.getPosInBox(box.func_186670_a(pos));
        for (BlockPos p : positions) {
            if (onAxis(pos, p, axis)) {
                IBlockState state = world.func_180495_p(p);
                if (state.func_177230_c() instanceof BlockAxle) {
                    continue;
                } else {
                    showErrorMessage(player, Error.SPACE);
                    return false;
                }
            }
            IBlockState state = world.func_180495_p(p);
            if (!state.func_185904_a().func_76222_j()) {
                return false;
            }
        }
        return !containsOtherGenerator(world, pos, axis);
    }

    private boolean onAxis(BlockPos base, BlockPos test, EnumFacing.Axis axis) {
        switch (axis) {
            case X:
                return base.func_177952_p() == test.func_177952_p() && base.func_177956_o() == test.func_177956_o();
            case Y:
                return base.func_177952_p() == test.func_177952_p() && base.func_177958_n() == test.func_177958_n();
            case Z:
                return base.func_177958_n() == test.func_177958_n() && base.func_177956_o() == test.func_177956_o();
            default:
                return false;
        }
    }

    @Nonnull
    @Override
    public EnumActionResult func_180614_a(EntityPlayer player, World worldIn, @Nonnull BlockPos pos, @Nonnull EnumHand hand, @Nonnull EnumFacing facing, float hitX, float hitY, float hitZ) {
        if (worldIn.field_72995_K)
            return EnumActionResult.PASS;
        EnumFacing.Axis axis = getAxleAxis(worldIn, pos);
        if (axis != null) {
            if (isValidArea(worldIn, player, pos, axis)) {
                worldIn.func_175656_a(pos, ((BlockAxleGenerator) field_150939_a).getAxisState(axis));
                return EnumActionResult.SUCCESS;
            } else {
                showErrorMessage(player, Error.SPACE);
            }
        } else {
            showErrorMessage(player, Error.AXLE);
        }
        return EnumActionResult.PASS;
    }

    private EnumFacing.Axis getAxleAxis(World world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos);
        if (state.func_177230_c() instanceof BlockAxle) {
            EnumFacing.Axis axis = state.func_177229_b(DirUtils.AXIS);
            return isAxis(axis) ? axis : null;
        }
        return null;
    }

    @Override
    public AxisAlignedBB getBounds(World world, BlockPos pos, EnumFacing facing, float flX, float flY, float flZ, ItemStack stack, EntityLivingBase placer) {
        return getBounds(facing.func_176740_k(), this.radius);
    }

    @SideOnly(Side.CLIENT)
    public enum Error {
        SPACE("bwm.message.%s.space"),
        AXLE("bwm.message.%s.axle");

        public final String key;

        Error(String key) {
            this.key = key;
        }

        public String format(String block) {
            return String.format(key, block);
        }
    }
}
