package betterwithmods.module.gameplay;

import betterwithmods.common.items.ItemMaterial;
import betterwithmods.module.Feature;
import betterwithmods.util.CapabilityUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.item.ItemStack;
import net.minecraft.util.DamageSource;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

public class BlastingOil extends Feature {

    private static HashMap<EntityItem, Double> highestPoint = Maps.newHashMap();
    public boolean disableBlastingOilEvents;
    public List<String> blacklistDamageSources;

    @Override
    public String getFeatureDescription() {
        return "Make blasting oil very dangerous";
    }

    @Override
    public void setupConfig() {
        disableBlastingOilEvents = loadPropBool("Disable Blasting Oil", "Don't process blasting oil explosions, as they are have major performance impact", false);
        blacklistDamageSources = Lists.newArrayList(loadPropStringList("Blasting oil damage source blacklist", "Disallow these damage sources from disturbing blasting oil", new String[]{
                "drown",
                "cramming",
                "generic",
                "wither",
                "starve",
                "outOfWorld"
        }));
    }

    @SubscribeEvent
    public void onPlayerTakeDamage(LivingHurtEvent e) {
        if (disableBlastingOilEvents)
            return;

        if (blacklistDamageSources.contains(e.getSource().field_76373_n))
            return;

        DamageSource BLAST_OIL = new DamageSource("blastingoil");
        EntityLivingBase living = e.getEntityLiving();
        if (living.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null)) {

            IItemHandler inventory = CapabilityUtils.getEntityInventory(living);
            int count = 0;
            for (int i = 0; i < inventory.getSlots(); i++) {
                ItemStack stack = inventory.getStackInSlot(i);

                if (!stack.func_190926_b() && stack.func_77969_a(ItemMaterial.getStack(ItemMaterial.EnumMaterial.BLASTING_OIL))) {
                    count += stack.func_190916_E();
                    inventory.extractItem(i, stack.func_190916_E(), false);
                }
            }
            if (count > 0) {
                living.func_70097_a(BLAST_OIL, Float.MAX_VALUE);
                living.func_130014_f_().func_72876_a(null, living.field_70165_t, living.field_70163_u + living.field_70131_O / 16, living.field_70161_v, (float) (Math.sqrt(count / 5) / 2.5 + 1), true);
            }
        }
    }

    @SubscribeEvent
    public void onHitGround(TickEvent.WorldTickEvent event) {
        if (disableBlastingOilEvents)
            return;
        List<EntityItem> items;
        World world = event.world;
        if (world.field_72995_K || event.phase != TickEvent.Phase.END)
            return;

        synchronized (world.field_72996_f) {
            items = world.field_72996_f.stream().filter(e -> e instanceof EntityItem && ((EntityItem) e).func_92059_d().func_77969_a(ItemMaterial.getStack(ItemMaterial.EnumMaterial.BLASTING_OIL))).map(e -> (EntityItem) e).collect(Collectors.toList());
        }
        HashSet<EntityItem> toRemove = new HashSet<>();
        items.forEach(item -> {
            boolean ground = item.field_70122_E;
            if (item.func_70027_ad() || (ground && Math.abs(item.field_70163_u - highestPoint.getOrDefault(item, item.field_70163_u)) > 2.0)) {
                int count = item.func_92059_d().func_190916_E();
                if (count > 0) {
                    world.func_72876_a(item, item.field_70165_t, item.field_70163_u + item.field_70131_O / 16, item.field_70161_v, (float) (Math.sqrt(count / 5) / 2.5 + 1), true);
                    toRemove.add(item);
                    item.func_70106_y();
                }
            }
            if (item.field_70181_x > 0 || !highestPoint.containsKey(item))
                highestPoint.put(item, item.field_70163_u);
        });
        toRemove.forEach(highestPoint::remove);
    }
}
