package betterwithmods.module.gameplay.miniblocks;

import betterwithmods.BWMod;
import betterwithmods.api.util.IBlockVariants;
import betterwithmods.client.model.render.RenderUtils;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMRecipes;
import betterwithmods.common.BWOreDictionary;
import betterwithmods.common.BWRegistry;
import betterwithmods.common.blocks.BlockAesthetic;
import betterwithmods.common.blocks.camo.BlockCamo;
import betterwithmods.common.items.ItemMaterial;
import betterwithmods.module.Feature;
import betterwithmods.module.gameplay.AnvilRecipes;
import betterwithmods.module.gameplay.miniblocks.blocks.*;
import betterwithmods.module.gameplay.miniblocks.client.CamoModel;
import betterwithmods.module.gameplay.miniblocks.client.MiniModel;
import betterwithmods.module.gameplay.miniblocks.client.StairModel;
import betterwithmods.util.JsonUtils;
import betterwithmods.util.ReflectionHelperBlock;
import com.google.common.collect.*;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import net.minecraft.block.*;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.IBakedModel;
import net.minecraft.client.renderer.block.model.ModelResourceLocation;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.Ingredient;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.registry.IRegistry;
import net.minecraft.world.World;
import net.minecraftforge.client.event.ModelBakeEvent;
import net.minecraftforge.common.crafting.CraftingHelper;
import net.minecraftforge.event.RegistryEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.ForgeRegistries;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.oredict.ShapedOreRecipe;

import java.io.File;
import java.nio.file.Files;
import java.util.*;
import java.util.stream.Collectors;


public class MiniBlocks extends Feature {
    public static final HashMap<MiniType, HashMap<Material, BlockCamo>> MINI_MATERIAL_BLOCKS = Maps.newHashMap();
    public static final Multimap<Material, IBlockState> MATERIALS = HashMultimap.create();
    private static final Map<Material, String> names = Maps.newHashMap();
    private static boolean autoGeneration;
    private static boolean requiresAnvil;
    private static Set<Ingredient> WHITELIST;


    static {
        for (MiniType type : MiniType.VALUES) {
            MINI_MATERIAL_BLOCKS.put(type, Maps.newHashMap());
        }
    }

    public MiniBlocks() {
        enabledByDefault = false;
    }

    public static List<ItemStack> getStacks(MiniType type, Material material) {
        List<ItemStack> stacks = Lists.newArrayList();
        Block block = MiniBlocks.MINI_MATERIAL_BLOCKS.get(type).get(material);
        for (IBlockState state : MiniBlocks.MATERIALS.get(material)) {
            stacks.add(MiniBlocks.fromParent(block, state));
        }
        return stacks;
    }

    public static boolean isValidMini(IBlockState state) {
        Material material = state.func_185904_a();
        return names.containsKey(material) && MATERIALS.get(material).contains(state);
    }

    public static boolean isValidMini(IBlockState state, ItemStack stack) {
        Block blk = state.func_177230_c();
        final ReflectionHelperBlock pb = new ReflectionHelperBlock();
        final Class<? extends Block> blkClass = blk.getClass();

        pb.func_180639_a(null, null, null, null, null, null, 0, 0, 0);
        boolean noActivation = (getDeclaringClass(blkClass, pb.MethodName, World.class, BlockPos.class, IBlockState.class, EntityPlayer.class, EnumHand.class, EnumFacing.class, float.class, float.class, float.class) == Block.class);

        pb.func_180650_b(null, null, null, null);
        boolean noUpdate = getDeclaringClass(blkClass, pb.MethodName, World.class, BlockPos.class, IBlockState.class, Random.class) == Block.class;

        // ignore blocks with custom collision.
        pb.func_180634_a(null, null, null, null);
        boolean noCustomCollision = getDeclaringClass(blkClass, pb.MethodName, World.class, BlockPos.class, IBlockState.class, Entity.class) == Block.class;
        final boolean isFullBlock = state.func_185913_b() || blkClass == BlockStainedGlass.class || blkClass == BlockGlass.class || blk == Blocks.field_180399_cE || blk == Blocks.field_150432_aD;
        final boolean hasItem = Item.func_150898_a(blk) != Items.field_190931_a;
        final boolean tickingBehavior = blk.func_149653_t();
        final boolean isOre = BWOreDictionary.hasPrefix(stack, "ore") || BWOreDictionary.isOre(stack, "logWood");

        boolean hasBehavior = (blk.hasTileEntity(state) || tickingBehavior) && blkClass != BlockGrass.class && blkClass != BlockIce.class;

        return noUpdate && noActivation && noCustomCollision && isFullBlock && !hasBehavior && hasItem && !isOre;
    }

    private static Class<?> getDeclaringClass(
            final Class<?> blkClass,
            final String methodName,
            final Class<?>... args) {
        try {
            blkClass.getDeclaredMethod(methodName, args);
            return blkClass;
        } catch (final NoSuchMethodException | SecurityException e) {
            // nothing here...
        } catch (final NoClassDefFoundError e) {
            BWMod.logger.info("Unable to determine blocks eligibility for making a miniblock, " + blkClass.getName() + " attempted to load " + e.getMessage());
            return blkClass;
        } catch (final Throwable t) {
            return blkClass;
        }

        return getDeclaringClass(
                blkClass.getSuperclass(),
                methodName,
                args);
    }

    public static ItemStack fromParent(Block mini, IBlockState state) {
        return fromParent(mini, state, 1);
    }

    public static ItemStack fromParent(Block mini, IBlockState state, int count) {
        ItemStack stack = new ItemStack(mini, count);
        NBTTagCompound tag = new NBTTagCompound();
        NBTTagCompound texture = new NBTTagCompound();
        NBTUtil.func_190009_a(texture, state);
        tag.func_74782_a("texture", texture);
        stack.func_77982_d(tag);
        return stack.func_77946_l();
    }

    public static void forceMiniBlock(Ingredient ingredient) {
        WHITELIST.add(ingredient);
    }

    public static void addMaterial(Material material, String name) {
        if (!names.containsKey(material)) //so addons don't overwrite our names, causing world breakage
            names.put(material, name);
    }

    @SideOnly(Side.CLIENT)
    public static void registerModel(IRegistry<ModelResourceLocation, IBakedModel> registry, String name, IBakedModel model) {
        registerModel(registry, name, model, Sets.newHashSet("normal", "inventory"));
    }

    @SideOnly(Side.CLIENT)
    public static void registerModel(IRegistry<ModelResourceLocation, IBakedModel> registry, String name, IBakedModel model, Set<String> variants) {
        for (String variant : variants) {
            registry.func_82595_a(new ModelResourceLocation(BWMod.MODID + ":" + name, variant), model);
        }
    }

    public Set<Ingredient> loadMiniblockWhitelist() {
        File file = new File(configHelper.path, "betterwithmods/miniblocks.json");

        //noinspection ResultOfMethodCallIgnored
        file.getParentFile().mkdirs();
        if (!Files.exists(file.toPath())) {
            JsonArray DEFAULT_CONFIG = new JsonArray();
            DEFAULT_CONFIG.add(JsonUtils.fromOre("plankWood"));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150347_e)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150348_b)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150417_aV)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150322_A)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_180395_cM)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_185767_cT)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150336_V)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150385_bj)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150371_ca)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150340_R)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(new ItemStack(Blocks.field_150339_S)));
            DEFAULT_CONFIG.add(JsonUtils.fromStack(BlockAesthetic.getStack(BlockAesthetic.EnumType.WHITESTONE)));
            JsonUtils.writeFile(file, DEFAULT_CONFIG);
        }
        JsonObject[] objects = JsonUtils.readerFile(file);
        if (objects != null)
            return Arrays.stream(objects).map(object -> CraftingHelper.getIngredient(object, JsonUtils.BWM_CONTEXT)).collect(Collectors.toSet());
        return Sets.newHashSet();
    }

    @Override
    public void preInit(FMLPreInitializationEvent event) {
        names.put(Material.field_151575_d, "wood");
        names.put(Material.field_151576_e, "rock");
        names.put(Material.field_151573_f, "iron");
        MiniType.registerTiles();
    }

    @SubscribeEvent(priority = EventPriority.HIGHEST)
    public void beforeBlockRegister(RegistryEvent.Register<Block> event) {

        for (Material material : names.keySet()) {
            String name = names.get(material);
            MINI_MATERIAL_BLOCKS.get(MiniType.SIDING).put(material, (BlockMini) new BlockSiding(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "siding", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.MOULDING).put(material, (BlockMini) new BlockMoulding(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "moulding", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.CORNER).put(material, (BlockMini) new BlockCorner(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "corner", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.COLUMN).put(material, (BlockMini) new BlockColumn(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "column", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.PEDESTAL).put(material, (BlockMini) new BlockPedestals(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "pedestal", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.STAIR).put(material, (BlockMini) new BlockStair(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "stair", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.TABLE).put(material, (BlockCamo) new BlockTable(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "table", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.BENCH).put(material, (BlockCamo) new BlockBench(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "bench", name)));
            MINI_MATERIAL_BLOCKS.get(MiniType.CHAIR).put(material, (BlockCamo) new BlockChair(material, MATERIALS::get).setRegistryName(String.format("%s_%s", "chair", name)));
        }

        for (MiniType type : MiniType.VALUES) {
            for (BlockCamo mini : MINI_MATERIAL_BLOCKS.get(type).values()) {
                BWMBlocks.registerBlock(mini, mini.createItemBlock(mini));
            }
        }
    }

    public void registerMiniblocks() {
        WHITELIST = loadMiniblockWhitelist();

        final NonNullList<ItemStack> list = NonNullList.func_191196_a();

        Iterable<Item> items = ForgeRegistries.ITEMS;
        if (!autoGeneration)
            items = WHITELIST.stream().map(Ingredient::func_193365_a).flatMap(Arrays::stream).map(ItemStack::func_77973_b).collect(Collectors.toSet());

        for (Item item : items) {
            if (!(item instanceof ItemBlock))
                continue;
            try {
                final CreativeTabs ctab = item.func_77640_w();
                if (ctab != null) {
                    item.func_150895_a(ctab, list);
                }
                for (final ItemStack stack : list) {
                    if (!(stack.func_77973_b() instanceof ItemBlock))
                        continue;
                    IBlockState state = BWMRecipes.getStateFromStack(stack);
                    if (state != null && isValidMini(state, stack)) {
                        Material material = state.func_185904_a();
                        if (names.containsKey(material)) {
                            MATERIALS.put(material, state);
                        }
                    }
                }
                list.clear();
            } catch (Throwable ignored) {
            }
        }
    }


    @SubscribeEvent
    public void registerRecipes(RegistryEvent.Register<IRecipe> event) {

        registerMiniblocks();

        BWOreDictionary.registerOre("miniblocks",
                new ItemStack(MiniBlocks.MINI_MATERIAL_BLOCKS.get(MiniType.SIDING).get(Material.field_151575_d)),
                new ItemStack(MiniBlocks.MINI_MATERIAL_BLOCKS.get(MiniType.MOULDING).get(Material.field_151575_d)),
                new ItemStack(MiniBlocks.MINI_MATERIAL_BLOCKS.get(MiniType.CORNER).get(Material.field_151575_d)));

        for (Material material : names.keySet()) {
            BlockCamo siding = MINI_MATERIAL_BLOCKS.get(MiniType.SIDING).get(material);
            BlockCamo moulding = MINI_MATERIAL_BLOCKS.get(MiniType.MOULDING).get(material);
            BlockCamo corner = MINI_MATERIAL_BLOCKS.get(MiniType.CORNER).get(material);

            event.getRegistry().register(new MiniRecipe(siding, null));
            event.getRegistry().register(new MiniRecipe(moulding, siding));
            event.getRegistry().register(new MiniRecipe(corner, moulding));
        }

        for (IBlockState parent : MATERIALS.values()) {
            ItemStack parentStack = BWMRecipes.getStackFromState(parent);
            Material material = parent.func_185904_a();
            MiniBlockIngredient siding = new MiniBlockIngredient("siding", parentStack);
            MiniBlockIngredient moulding = new MiniBlockIngredient("moulding", parentStack);

            ItemStack columnStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.COLUMN).get(material), parent, 8);
            ItemStack pedestalStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.PEDESTAL).get(material), parent, 8);
            ItemStack tableStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.TABLE).get(material), parent, 1);
            ItemStack benchStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.BENCH).get(material), parent, 1);
            ItemStack chairStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.CHAIR).get(material), parent, 2);

            AnvilRecipes.addSteelShapedRecipe(columnStack.func_77973_b().getRegistryName(), columnStack, "XX", "XX", "XX", "XX", 'X', moulding);
            AnvilRecipes.addSteelShapedRecipe(pedestalStack.func_77973_b().getRegistryName(), pedestalStack, " XX ", "BBBB", "BBBB", "BBBB", 'X', siding, 'B', parentStack);

            event.getRegistry().register(new ShapedOreRecipe(chairStack.func_77973_b().getRegistryName(), chairStack, "  S", "SSS", "M M", 'S', siding, 'M', moulding).setMirrored(true).setRegistryName(getRecipeRegistry(chairStack, parentStack)));
            event.getRegistry().register(new ShapedOreRecipe(tableStack.func_77973_b().getRegistryName(), tableStack, "SSS", " M ", " M ", 'S', siding, 'M', moulding).setRegistryName(getRecipeRegistry(tableStack, parentStack)));
            event.getRegistry().register(new ShapedOreRecipe(benchStack.func_77973_b().getRegistryName(), benchStack, "SSS", " M ", 'S', siding, 'M', moulding).setRegistryName(getRecipeRegistry(benchStack, parentStack)));

            IBlockVariants blockVariants = BWOreDictionary.getVariantFromState(IBlockVariants.EnumBlock.BLOCK, parent);
            if (blockVariants != null) {
                ItemStack fence = blockVariants.getVariant(IBlockVariants.EnumBlock.FENCE, 2);
                ItemStack fencegate = blockVariants.getVariant(IBlockVariants.EnumBlock.FENCE_GATE, 1);
                ItemStack stairs = blockVariants.getVariant(IBlockVariants.EnumBlock.STAIR, 1);
                ItemStack wall = blockVariants.getVariant(IBlockVariants.EnumBlock.WALL, 3);
                if (!wall.func_190926_b())
                    event.getRegistry().register(new ShapedOreRecipe(wall.func_77973_b().getRegistryName(), wall, "SSS", 'S', siding).setRegistryName(getRecipeRegistry(wall, parentStack)));
                if (!stairs.func_190926_b())
                    event.getRegistry().register(new ShapedOreRecipe(stairs.func_77973_b().getRegistryName(), stairs, "M ", "MM", 'M', moulding).setMirrored(true).setRegistryName(getRecipeRegistry(stairs, parentStack)));
                if (!fence.func_190926_b())
                    event.getRegistry().register(new ShapedOreRecipe(fence.func_77973_b().getRegistryName(), fence, "MMM", 'M', moulding).setRegistryName(getRecipeRegistry(fence, parentStack)));
                if (!fencegate.func_190926_b())
                    event.getRegistry().register(new ShapedOreRecipe(fencegate.func_77973_b().getRegistryName(), fencegate, "MSM", 'M', moulding, 'S', siding).setRegistryName(getRecipeRegistry(fencegate, parentStack)));
            }

            if (!requiresAnvil || material == Material.field_151575_d) {
                MiniBlockIngredient corner = new MiniBlockIngredient("corner", parentStack);
                ItemStack sidingStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.SIDING).get(material), parent, 2);
                ItemStack mouldingStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.MOULDING).get(material), parent, 2);
                ItemStack cornerStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.CORNER).get(material), parent, 2);
                BWRegistry.WOOD_SAW.addRecipe(parentStack, sidingStack);
                BWRegistry.WOOD_SAW.addRecipe(siding, mouldingStack);
                BWRegistry.WOOD_SAW.addRecipe(moulding, cornerStack);
                if (BWOreDictionary.isOre(parentStack, "plankWood")) {
                    BWRegistry.WOOD_SAW.addRecipe(corner, ItemMaterial.getStack(ItemMaterial.EnumMaterial.WOOD_GEAR, 2));
                }
            } else {
                ItemStack sidingStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.SIDING).get(material), parent, 8);
                ItemStack mouldingStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.MOULDING).get(material), parent, 8);
                ItemStack cornerStack = MiniBlocks.fromParent(MINI_MATERIAL_BLOCKS.get(MiniType.CORNER).get(material), parent, 8);

                AnvilRecipes.addSteelShapedRecipe(sidingStack.func_77973_b().getRegistryName(), sidingStack, "XXXX", 'X', parentStack);
                AnvilRecipes.addSteelShapedRecipe(mouldingStack.func_77973_b().getRegistryName(), mouldingStack, "XXXX", 'X', siding);
                AnvilRecipes.addSteelShapedRecipe(cornerStack.func_77973_b().getRegistryName(), cornerStack, "XXXX", 'X', moulding);
            }
        }

    }

    private ResourceLocation getRecipeRegistry(ItemStack output, ItemStack parent) {
        if (parent.func_77960_j() > 0)
            return new ResourceLocation(BWMod.MODID, output.func_77973_b().getRegistryName().func_110623_a() + "_" + parent.func_77973_b().getRegistryName().func_110623_a() + "_" + parent.func_77960_j());
        return new ResourceLocation(BWMod.MODID, output.func_77973_b().getRegistryName().func_110623_a() + "_" + parent.func_77973_b().getRegistryName().func_110623_a());
    }

    @Override
    public void setupConfig() {
        autoGeneration = loadPropBool("Auto Generate Miniblocks", "Automatically add miniblocks for many blocks, based on heuristics and probably planetary alignments. WARNING: Exposure to this config option can kill pack developers.", false);
        requiresAnvil = loadPropBool("Stone Miniblocks require Anvil recipe", "When enabled stone and metal miniblocks will require an anvil recipe, when disabled they will all be made with the saw", true);
    }

    @Override
    public String getFeatureDescription() {
        return "Dynamically generate Siding, Mouldings and Corners for many of the blocks in the game.";
    }


    @Override
    public boolean hasSubscriptions() {
        return true;
    }

    @SubscribeEvent
    @SideOnly(Side.CLIENT)
    public void onPostBake(ModelBakeEvent event) {
        MiniModel.SIDING = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/siding")));
        MiniModel.MOULDING = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/moulding")));
        MiniModel.CORNER = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/corner")));
        MiniModel.COLUMN = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/column")));
        MiniModel.PEDESTAL = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/pedestal")));
        MiniModel.STAIR = new StairModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/stair")),
                RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/mini/stair_inner_corner")));
        MiniModel.CHAIR = new MiniModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/chair")));

        CamoModel.TABLE_SUPPORTED = new CamoModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/table_supported")));
        CamoModel.TABLE_UNSUPPORTED = new CamoModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/table_unsupported")));

        CamoModel.BENCH_SUPPORTED = new CamoModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/bench_supported")));
        CamoModel.BENCH_UNSUPPORTED = new CamoModel(RenderUtils.getModel(new ResourceLocation(BWMod.MODID, "block/bench_unsupported")));

        for (Material material : names.keySet()) {
            String name = names.get(material);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "siding", name), MiniModel.SIDING);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "moulding", name), MiniModel.MOULDING);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "corner", name), MiniModel.CORNER);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "column", name), MiniModel.COLUMN);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "pedestal", name), MiniModel.PEDESTAL);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "stair", name), MiniModel.STAIR);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "chair", name), MiniModel.CHAIR);
            registerModel(event.getModelRegistry(), String.format("%s_%s", "table", name), CamoModel.TABLE_SUPPORTED, Sets.newHashSet("normal", "inventory", "supported=true"));
            registerModel(event.getModelRegistry(), String.format("%s_%s", "table", name), CamoModel.TABLE_UNSUPPORTED, Sets.newHashSet("supported=false"));
            registerModel(event.getModelRegistry(), String.format("%s_%s", "bench", name), CamoModel.BENCH_SUPPORTED, Sets.newHashSet("normal", "inventory", "supported=true"));
            registerModel(event.getModelRegistry(), String.format("%s_%s", "bench", name), CamoModel.BENCH_UNSUPPORTED, Sets.newHashSet("supported=false"));

        }
    }

}
